@@ -60,6 +60,9 @@ extern struct servent *hes_getservbyname (/* char *, char * */);
 
 #ifdef SOCKS
 #include <socks.h>
+/* This doesn't belong here. */
+struct tm *localtime(const time_t *);
+struct hostent  *gethostbyname(const char *);
 #endif
 
 #include <des.h>
@@ -68,7 +71,7 @@ extern struct servent *hes_getservbyname (/* char *, char * */);
 #include <roken.h>
 
 static int socket_connection ( char *, int );
-static char *getline ( popserver );
+static char *pop_getline ( popserver );
 static int sendline ( popserver, char * );
 static int fullwrite ( int, char *, int );
 static int getok ( popserver );
@@ -77,12 +80,6 @@ static int gettermination ( popserver );
 #endif
 static void pop_trash ( popserver );
 static char *find_crlf ( char * );
-static char *getline (popserver server);
-static int sendline (popserver server, char *line);
-static int fullwrite (int fd, char *buf, int nbytes);
-static int getok (popserver server);
-static void pop_trash (popserver server);
-static char *find_crlf (char *in_string);
 #if defined(__ultrix) || IRIX == 4
 char *getpass(const char*);
 #endif
@@ -304,7 +301,7 @@ pop_stat (server, count, size)
       return (-1);
     }
      
-  if (sendline (server, "STAT") || (! (fromserver = getline (server))))
+  if (sendline (server, "STAT") || (! (fromserver = pop_getline (server))))
     return (-1);
 
   if (strncmp (fromserver, "+OK ", 4))
@@ -400,7 +397,7 @@ pop_list (server, message, IDs, sizes)
 	  free (*sizes);
 	  return (-1);
 	}
-      if (! (fromserver = getline (server)))
+      if (! (fromserver = pop_getline (server)))
 	{
 	  free (*IDs);
 	  free (*sizes);
@@ -646,7 +643,7 @@ pop_multi_first (server, command, response)
       return (-1);
     }
 
-  if (sendline (server, command) || (! (*response = getline (server))))
+  if (sendline (server, command) || (! (*response = pop_getline (server))))
     {
       return (-1);
     }
@@ -683,7 +680,7 @@ pop_multi_next (server, line)
       return (-1);
     }
 
-  fromserver = getline (server);
+  fromserver = pop_getline (server);
   if (! fromserver)
     {
       return (-1);
@@ -820,7 +817,7 @@ pop_last (server)
   if (sendline (server, "LAST"))
     return (-1);
 
-  if (! (fromserver = getline (server)))
+  if (! (fromserver = pop_getline (server)))
     return (-1);
 
   if (! strncmp (fromserver, "-ERR", 4))
@@ -945,10 +942,7 @@ socket_connection (char *host, int flags)
       hostent = gethostbyname (host);
       try_count++;
       if ((! hostent)
-#ifdef HAVE_H_ERRNO
-	  && ((h_errno != TRY_AGAIN) || (try_count == 5))
-#endif
-	  )
+	  && ((h_errno != TRY_AGAIN) || (try_count == 5)))
 	{
 	  strcpy (pop_error, "Could not determine POP server's address");
 	  return (-1);
@@ -1014,14 +1008,16 @@ socket_connection (char *host, int flags)
 #define KRB_ERROR "Kerberos error connecting to POP server: "
   if (! (flags & POP_NO_KERBEROS))
     {
+      char *hostname = strdup((char *)hostent->h_name);
       ticket = (KTEXT) malloc (sizeof (KTEXT_ST));
-      rem = krb_sendauth (0L, sock, ticket, "pop", (char *)hostent->h_name,
-			  (char *) krb_realmofhost (hostent->h_name),
-			  (unsigned long) 0, &msg_data, &cred, schedule,
+      rem = krb_sendauth (0L, sock, ticket, "pop", hostname,
+			  krb_realmofhost (hostname),
+			  0UL, &msg_data, &cred, schedule,
 			  (struct sockaddr_in *) 0,
 			  (struct sockaddr_in *) 0,
 			  "KPOPV0.1");
       free (ticket);
+      free (hostname);
       if (rem != KSUCCESS)
 	{
 	  strcpy (pop_error, KRB_ERROR);
@@ -1036,7 +1032,7 @@ socket_connection (char *host, int flags)
 } /* socket_connection */
 
 /*
- * Function: getline
+ * Function: pop_getline
  *
  * Purpose: Get a line of text from the connection and return a
  * 	pointer to it.  The carriage return and linefeed at the end of
@@ -1049,12 +1045,12 @@ socket_connection (char *host, int flags)
  * Returns: A non-null pointer if successful, or a null pointer on any
  * 	error, with an error message copied into pop_error.
  *
- * Notes: The line returned is overwritten with each call to getline.
+ * Notes: The line returned is overwritten with each call to pop_getline.
  *
  * Side effects: Closes the connection on error.
  */
 static char *
-getline (popserver server)
+pop_getline (popserver server)
 {
 #define GETLINE_ERROR "Error reading from server: "
 
@@ -1111,7 +1107,7 @@ getline (popserver server)
 	  server->buffer = realloc (server->buffer, server->buffer_size);
 	  if (! server->buffer)
 	    {
-	      strcpy (pop_error, "Out of memory in getline");
+	      strcpy (pop_error, "Out of memory in pop_getline");
 	      pop_trash (server);
 	      return (0);
 	    }
@@ -1128,7 +1124,7 @@ getline (popserver server)
 	}
       else if (ret == 0)
 	{
-	  strcpy (pop_error, "Unexpected EOF from server in getline");
+	  strcpy (pop_error, "Unexpected EOF from server in pop_getline");
 	  pop_trash (server);
 	  return (0);
 	}
@@ -1245,7 +1241,7 @@ getok (popserver server)
 {
   char *fromline;
 
-  if (! (fromline = getline (server)))
+  if (! (fromline = pop_getline (server)))
     {
       return (-1);
     }
@@ -1284,7 +1280,7 @@ gettermination (server)
 {
   char *fromserver;
 
-  fromserver = getline (server);
+  fromserver = pop_getline (server);
   if (! fromserver)
     return (-1);
 
