@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1998, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -39,17 +34,11 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 
-RCSID("$Id: md5.c,v 1.6 1997/10/14 15:16:32 joda Exp $");
+RCSID("$Id: md5.c,v 1.12 1999/12/02 16:58:38 joda Exp $");
 #endif
 
-#include <stdlib.h>
-#include <string.h>
-
 #include "md5.h"
-
-#ifndef min
-#define min(a,b) (((a)>(b))?(b):(a))
-#endif
+#include "hash.h"
 
 #define A m->counter[0]
 #define B m->counter[1]
@@ -68,16 +57,10 @@ md5_init (struct md5 *m)
   A = 0x67452301;
 }
 
-static inline u_int32_t
-cshift (u_int32_t x, unsigned int n)
-{
-  return (x << n) | (x >> (32 - n));
-}
-
-#define F(x,y,z) ((x & y) | (~x & z))
-#define G(x,y,z) ((x & z) | (y & ~z))
+#define F(x,y,z) CRAYFIX((x & y) | (~x & z))
+#define G(x,y,z) CRAYFIX((x & z) | (y & ~z))
 #define H(x,y,z) (x ^ y ^ z)
-#define I(x,y,z) (y ^ (x | ~z))
+#define I(x,y,z) CRAYFIX(y ^ (x | ~z))
 
 #define DOIT(a,b,c,d,k,s,i,OP) \
 a = b + cshift(a + OP(b,c,d) + X[k] + (i), s)
@@ -195,23 +178,20 @@ calc (struct md5 *m, u_int32_t *data)
  * From `Performance analysis of MD5' by Joseph D. Touch <touch@isi.edu>
  */
 
+#if defined(WORDS_BIGENDIAN)
 static inline u_int32_t
 swap_u_int32_t (u_int32_t t)
 {
-#if defined(WORDS_BIGENDIAN)
-#define ROL(x,n) ((x)<<(n))|((x)>>(32-(n)))
   u_int32_t temp1, temp2;
 
-  temp1   = ROL(t,16);
+  temp1   = cshift(t, 16);
   temp2   = temp1 >> 8;
   temp1  &= 0x00ff00ff;
   temp2  &= 0x00ff00ff;
   temp1 <<= 8;
   return temp1 | temp2;
-#else
-  return t;
-#endif
 }
+#endif
 
 struct x32{
   unsigned int a:32;
@@ -264,7 +244,7 @@ md5_finito (struct md5 *m, void *res)
   md5_update (m, zeros, dstart + 8);
   {
       int i;
-      u_char *r = (u_char *)res;
+      unsigned char *r = (unsigned char *)res;
 
       for (i = 0; i < 4; ++i) {
 	  r[4*i]   = m->counter[i] & 0xFF;
@@ -300,7 +280,7 @@ MD5Init (MD5_CTX *mdContext)
 void
 MD5Update (MD5_CTX *mdContext, const unsigned char *inBuf, unsigned int inLen)
 {
-  md5_update(&mdContext->m.d5, (unsigned char *)inBuf, inLen);
+  md5_update(&mdContext->m.d5, (const unsigned char *)inBuf, inLen);
 }
 
 void
