@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995, 1996, 1997, 1999 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -39,7 +34,7 @@
 #ifdef HAVE_CONFIG_H
 #include "config.h"
 
-RCSID("$Id: rnd_keys.c,v 1.53 1998/02/24 14:26:08 bg Exp $");
+RCSID("$Id: rnd_keys.c,v 1.56 1999/12/02 16:58:39 joda Exp $");
 #endif
 
 #include <des.h>
@@ -228,6 +223,21 @@ pacemaker(struct timeval *tv)
 }
 #endif
 
+#ifdef HAVE_SIGACTION
+/* XXX ugly hack, should perhaps use function from roken */
+static RETSIGTYPE 
+(*fake_signal(int sig, RETSIGTYPE (*f)(int)))(int)
+{
+    struct sigaction sa, osa;
+    sa.sa_handler = f;
+    sa.sa_flags = 0;
+    sigemptyset(&sa.sa_mask);
+    sigaction(sig, &sa, &osa);
+    return osa.sa_handler;
+}
+#define signal(S, F) fake_signal((S), (F))
+#endif
+
 /*
  * Generate size bytes of "random" data using timed interrupts.
  * It takes about 40ms/byte random data.
@@ -237,13 +247,10 @@ void
 des_rand_data(unsigned char *data, int size)
 {
     struct itimerval tv, otv;
-#ifdef HAVE_SIGACTION
-    struct sigaction sa, osa;
-#else
     RETSIGTYPE (*osa)(int);
-#endif
     int i, j;
-#ifndef HAVE_SETITIMER
+#ifndef HAVE_SETITIMER 
+    RETSIGTYPE (*ochld)(int);
     pid_t pid;
 #endif
     char *rnd_devices[] = {"/dev/random",
@@ -270,15 +277,7 @@ des_rand_data(unsigned char *data, int size)
     gsize = size;
     igdata = 0;
 
-#ifdef HAVE_SIGACTION
-    /* Setup signal handler */
-    sa.sa_handler = sigALRM;
-    sa.sa_flags = 0;
-    sigemptyset(&sa.sa_mask);
-    sigaction(SIGALRM, &sa, &osa);
-#else
     osa = signal(SIGALRM, sigALRM);
-#endif
   
     /* Start timer */
     tv.it_value.tv_sec = 0;
@@ -287,8 +286,10 @@ des_rand_data(unsigned char *data, int size)
 #ifdef HAVE_SETITIMER
     setitimer(ITIMER_REAL, &tv, &otv);
 #else
+    ochld = signal(SIGCHLD, SIG_IGN);
     pid = fork();
     if(pid == -1){
+	signal(SIGCHLD, ochld != SIG_ERR ? ochld : SIG_DFL);
 	des_not_rand_data(data, size);
 	return;
     }
@@ -306,13 +307,10 @@ des_rand_data(unsigned char *data, int size)
     setitimer(ITIMER_REAL, &otv, 0);
 #else
     kill(pid, SIGKILL);
-    waitpid(pid, NULL, 0);
+    while(waitpid(pid, NULL, 0) != pid);
+    signal(SIGCHLD, ochld != SIG_ERR ? ochld : SIG_DFL);
 #endif
-#ifdef HAVE_SIGACTION
-    sigaction(SIGALRM, &osa, 0);
-#else
     signal(SIGALRM, osa != SIG_ERR ? osa : SIG_DFL);
-#endif
 }
 #else
 void
