@@ -1,5 +1,5 @@
 /*
- * Copyright (c) 1995, 1996, 1997, 1998 Kungliga Tekniska Hgskolan
+ * Copyright (c) 1995 - 2000 Kungliga Tekniska Hgskolan
  * (Royal Institute of Technology, Stockholm, Sweden).
  * All rights reserved.
  * 
@@ -33,7 +33,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: kdc_reply.c,v 1.12 1999/12/02 16:58:42 joda Exp $");
+RCSID("$Id: kdc_reply.c,v 1.12.2.1 2000/06/23 03:30:42 assar Exp $");
 
 static int little_endian; /* XXX ugly */
 
@@ -115,7 +115,10 @@ kdc_reply_cipher(KTEXT reply, KTEXT cip)
     if(type != AUTH_MSG_KDC_REPLY)
 	return INTK_PROT;
 
-    p += krb_get_nir(p, aname, inst, realm);
+    p += krb_get_nir(p,
+		     aname, sizeof(aname),
+		     inst, sizeof(inst),
+		     realm, sizeof(realm));
     p += krb_get_int(p, &kdc_time, 4, little_endian);
     p++; /* number of tickets */
     p += krb_get_int(p, &exp_date, 4, little_endian);
