@@ -14,12 +14,7 @@
  *    notice, this list of conditions and the following disclaimer in the
  *    documentation and/or other materials provided with the distribution.
  * 
- * 3. All advertising materials mentioning features or use of this software
- *    must display the following acknowledgement:
- *      This product includes software developed by the Kungliga Tekniska
- *      Hgskolan and its contributors.
- * 
- * 4. Neither the name of the Institute nor the names of its contributors
+ * 3. Neither the name of the Institute nor the names of its contributors
  *    may be used to endorse or promote products derived from this software
  *    without specific prior written permission.
  * 
@@ -38,7 +33,7 @@
 
 #include "krb_locl.h"
 
-RCSID("$Id: mk_auth.c,v 1.6 1998/06/09 19:25:22 joda Exp $");
+RCSID("$Id: mk_auth.c,v 1.8 1999/12/02 16:58:43 joda Exp $");
 
 /*
  * Generate an authenticator for service.instance@realm.
@@ -69,7 +64,7 @@ krb_mk_auth(int32_t options,
   else
     tmp = krb_get_phost (instance);
 
-  strcpy_truncate(realinst, tmp, sizeof(realinst));
+  strlcpy(realinst, tmp, sizeof(realinst));
 
   if (realm == NULL) {
     ret = krb_get_lrealm (realrealm, 1);
